import 'package:bolter/bolter.dart';
import 'package:bolter_flutter/bolter_flutter.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

class BolterProvider<F> extends InheritedWidget {
  BolterProvider(
    this._useCaseContainer, {
    Key key,
    Widget child,
    this.flutterState,
  }) : super(key: key, child: child);

  final UseCaseContainer _useCaseContainer;
  final F flutterState;
  final _bolter = Bolter();

  void updateUI() => _bolter.shake();

  U useCase<U>({bool isSingleton = true}) =>
      _useCaseContainer.useCase<U>(isSingleton: isSingleton);

  ValueStream<T> stream<T>(Getter<T> getter, {bool distinct = true}) =>
      _bolter.stream(getter, distinct: distinct);

  ValueStream<T> streamFromUseCase<T, U>(T Function(U usecase) getter,
          {bool distinct = true, bool isSingleton = true}) =>
      _bolter.stream(() => getter(useCase(isSingleton: isSingleton)),
          distinct: distinct);

  static BolterProvider of(BuildContext context) =>
      context.dependOnInheritedWidgetOfExactType(aspect: BolterProvider);

  @override
  bool updateShouldNotify(BolterProvider oldWidget) => false;
}
